use std::cell::RefCell;

use crate::widgets::card_display::*;

use glib::subclass::InitializingObject;
use gtk::glib::clone;
use gtk::subclass::prelude::*;
use gtk::{glib, Box, Button, CompositeTemplate, Label};
use gtk::{prelude::*, Entry};

#[derive(CompositeTemplate, Default)]
#[template(resource = "/org/foxarmy/learn-hieroglyph/cards/game/ui.xml")]
pub struct MemoryCardsGameScene {
    #[template_child]
    pub card_display: TemplateChild<CardDisplay>,
    #[template_child]
    pub stats_label: TemplateChild<Label>,
    #[template_child]
    pub content: TemplateChild<Box>,
    #[template_child]
    pub no_cards_label: TemplateChild<Label>,
    #[template_child]
    pub back_button: TemplateChild<Button>,

    pub correct: RefCell<usize>,
    pub incorrect: RefCell<usize>,
}

#[glib::object_subclass]
impl ObjectSubclass for MemoryCardsGameScene {
    const NAME: &'static str = "MemoryCardsGameScene";
    type Type = super::MemoryCardsGameScene;
    type ParentType = gtk::ApplicationWindow;

    fn class_init(klass: &mut Self::Class) {
        klass.bind_template();
    }

    fn instance_init(obj: &InitializingObject<Self>) {
        obj.init_template();
    }
}

impl ObjectImpl for MemoryCardsGameScene {
    fn constructed(&self) {
        self.parent_constructed();

        let card_display_binding = self.card_display.imp().obj();
        let self_binding = self.obj();

        self_binding.update_card_list();

        card_display_binding.get_answer_entry().connect_activate(clone!(@strong card_display_binding, @strong self_binding  => move |e: &Entry| {
            println!("{}", e.text());
            println!("{} = {}? -> {}", e.text(), card_display_binding.get_hieroglyph(), e.text().to_string() == *card_display_binding.get_hieroglyph());
            if e.text() == *card_display_binding.get_hieroglyph() {
                *self_binding.imp().correct.borrow_mut() += 1;
            } else {
                *self_binding.imp().incorrect.borrow_mut() +=1;
            }
            self_binding.update_stats();
            self_binding.update_card_list();
            e.set_text("");
        }));
    }
}

impl WidgetImpl for MemoryCardsGameScene {}

impl WindowImpl for MemoryCardsGameScene {}

impl ApplicationWindowImpl for MemoryCardsGameScene {}
